@@ -1,5 +1,5 @@
 /*
- *  Copyright 1999-2001 Vizdom Software, Inc. All Rights Reserved.
+ *  Copyright 1999-2001,2003-2005,2008 Vizdom Software, Inc. All Rights Reserved.
  * 
  *  This program is free software; you can redistribute it and/or 
  *  modify it under the same terms as the Perl Kit, namely, under 
@@ -32,7 +32,7 @@ import com.vizdom.util.UnreachableCodeException;
  * A BER identifier, which consists of the tag (class and number) and form.
  * 
  * @author John Lacey
- * @version $Revision: 1.17 $
+ * @version $Revision: 1.18 $
  */
 public class BerIdentifier
 {
@@ -106,7 +106,7 @@ public class BerIdentifier
                 return BerSequence.gIDENTIFIER;
 
             default:
-                throw new UnreachableCodeException();
+                throw new UnreachableCodeException(tagNumber);
             }
         }
         else
