@@ -27,7 +27,7 @@ import com.vizdom.ber.BerModule;
  * as well as a BerModule instance for use by classes in this package.
  *
  * @author Gennis Emerson
- * @version $Revision: 1.13 $
+ * @version $Revision: 1.14 $
  */
 class BerDbdModule extends BerModule implements Cloneable
 {
@@ -169,6 +169,12 @@ class BerDbdModule extends BerModule implements Cloneable
     static final int gSTATEMENT_FUNC_RESPONSE = 
         gSTATEMENT_FUNC_REQUEST + sGAP;
 
+    /** A request. */
+    static final int gGET_GENERATED_KEYS_REQUEST =       (int) 0x21;
+    /** A response. */
+    static final int gGET_GENERATED_KEYS_RESPONSE = 
+        gGET_GENERATED_KEYS_REQUEST + sGAP;
+
     /** A module instance for use by classes in this package. */
     static final BerDbdModule gBerModule = new BerDbdModule();
 
@@ -208,6 +214,8 @@ class BerDbdModule extends BerModule implements Cloneable
 
         gBerModule.registerFactory(StatementFuncRequest.gFACTORY, 
             StatementFuncRequest.gIDENTIFIER);
+        gBerModule.registerFactory(GetGeneratedKeysRequest.gFACTORY, 
+            GetGeneratedKeysRequest.gIDENTIFIER);
 
         gBerModule.registerFactory(BerHash.gFACTORY, 
             BerHash.gIDENTIFIER);
