@@ -42,13 +42,13 @@ import org.apache.log4j.NDC;
  * intended to be run as a single thread started by a
  * connection-accepting server.
  *
- * @version $Revision: 1.38 $
+ * @version $Revision: 1.42 $
  */
 /* The handleMessage methods may throw only SQLExceptions and 
  * RuntimeExceptions, the first if they can recover, the second
  * if the connection should exit.
  */
-class Connection implements Runnable
+public class Connection implements Runnable
 {
     /** The log4j logger. */
     private static final Logger gLog = Logger.getLogger(Connection.class);
@@ -94,7 +94,7 @@ class Connection implements Runnable
     private boolean preExistingConnection;
 
     /** A collection of the Statements currently in use. */
-    private Hashtable mStatementTable;
+    private Hashtable mStatementTable = new Hashtable();
 
     /** 
      * The count of Statements created. This value is used as the
@@ -103,6 +103,23 @@ class Connection implements Runnable
     /* Starts at 1 in case we want the statement handle to be true in perl. */
     private int mNextHandle;
 
+    /** The user name. */
+    String mUser;
+
+    /** The password. */
+    String mPassword;
+
+    /** The JDBC URL. */
+    String mUrl;
+
+    /** Other connection properties. */
+    Properties mProperties;
+        
+    /** Flag indicating whether generated keys are supported by the driver. */
+    private boolean mSupportsGetGeneratedKeys; 
+
+    /** The cached generated keys. */
+    private GeneratedKey[] mGeneratedKeys; 
 
     /**
      * Constructor - initializes fields.
@@ -119,13 +136,11 @@ class Connection implements Runnable
 
         mIn = new BufferedInputStream(mSocket.getInputStream());
         mOut = new BufferedOutputStream(mSocket.getOutputStream());
-        mStatementTable = new Hashtable();
         mNextHandle = 1;
         mBerModule = aBerModule;
         preExistingConnection = false;
     }
 
-
     /**
      * Constructor. Uses a pre-existing JDBC connection.
      *
@@ -145,7 +160,6 @@ class Connection implements Runnable
         preExistingConnection = true;
     }
 
-
     /**
      * Accepts requests from client and dispatches them to the appropriate
      * method for handling. Sends response to client on request
@@ -181,7 +195,7 @@ class Connection implements Runnable
 
                 BerIdentifier id = request.getIdentifier(); 
                 if (id.getTagClass() != BerTypes.APPLICATION)
-                    throw new FatalException("Unknown request received");
+                    throw new FatalException("Unknown request received " + id);
 
                 int tagNumber = id.getTagNumber();
                 switch (tagNumber)
@@ -256,6 +270,11 @@ class Connection implements Runnable
                         (StatementFuncRequest) request); 
                     break;
 
+                case BerDbdModule.gGET_GENERATED_KEYS_REQUEST:
+                    response = handleRequest(
+                        (GetGeneratedKeysRequest) request);
+                    break;
+                    
                 default: 
                     throw new DbdException(DbdException.gUNKNOWN_REQUEST,
                         new String[] { String.valueOf(tagNumber) });
@@ -287,6 +306,8 @@ class Connection implements Runnable
             catch (Throwable throwable)
             {
                 connected = false;
+                gLog.warn("Rollback due to fatal error"); 
+                mRollback();
                 gLog.fatal("Error; ending connection", throwable);
                 try
                 {
@@ -313,6 +334,26 @@ class Connection implements Runnable
         NDC.pop();
     }
 
+    /**
+     * Attempts to rollback any existing transaction without
+     * triggering any errors (in case, for example, no
+     * transaction is in process). Intended to be used when
+     * ending the connection due to an error.
+     */
+    private void mRollback()
+    {
+        if (mConn == null)
+            return;
+        try
+        {
+            mConn.rollback();
+        }
+        catch (SQLException e)
+        {
+            gLog.warn("SQLException during rollback", e); 
+        }
+    }
+
     /**
      * Sets up the connection's character encoding, if the client 
      * requested a specific encoding. If this Connection was not
@@ -357,25 +398,36 @@ class Connection implements Runnable
 
             if (!preExistingConnection)
             {
-                String url =  aRequest.getURL(charset);
-                String user = aRequest.getUser(charset);
-                String password = aRequest.getPassword(charset);
-                Properties connProperties = aRequest.getProperties(charset);
+                mUrl =  aRequest.getURL(charset);
+                mUser = aRequest.getUser(charset);
+                mPassword = aRequest.getPassword(charset);
+                mProperties = aRequest.getProperties(charset);
                 if (gLog.isDebugEnabled())
                 {
-                    gLog.debug("url = " + aRequest.getURL(charset));
-                    gLog.debug("user = " + aRequest.getUser(charset));
-                    java.util.Enumeration keys = connProperties.keys();
+                    gLog.debug("url = " + mUrl);
+                    gLog.debug("user = " + mUser);
+                    java.util.Enumeration keys = mProperties.keys();
                     while (keys.hasMoreElements())
                     {
                         String key = (String) keys.nextElement();
-                        gLog.debug(key + " = " + connProperties.get(key));
+                        gLog.debug(key + " = " + mProperties.get(key));
                     }
                 }
-                if (user == null && password == null)
-                    mConn = DriverManager.getConnection(url, connProperties);
+                if (mUser == null && mPassword == null)
+                    mConn = DriverManager.getConnection(mUrl, mProperties);
                 else
-                    mConn = DriverManager.getConnection(url, user, password);
+                    mConn = DriverManager.getConnection(mUrl, mUser, mPassword);
+
+                DatabaseMetaData dbmd = mConn.getMetaData(); 
+                gLog.debug("Created database connection to " + 
+                    dbmd.getDatabaseProductName() + " v" + 
+                    dbmd.getDatabaseProductVersion()); 
+                mSupportsGetGeneratedKeys = dbmd.supportsGetGeneratedKeys(); 
+                if (gLog.isDebugEnabled())
+                {
+                    gLog.debug("Driver supports getGeneratedKeys? " + 
+                        mSupportsGetGeneratedKeys);
+                }
             }
         }
         catch (UnsupportedEncodingException unsupEnc)
@@ -385,7 +437,23 @@ class Connection implements Runnable
         }
         return new ConnectResponse();
     }
-    
+
+    /**
+     * Closes and reopens the underlying JDBC connection using
+     * the same connection properties. This is not currently
+     * exposed via DBD::JDBC; it was part of an experiment with
+     * exception listeners.
+     *
+     * @throws SQLException if a database error occurs
+     */
+    public void reconnect() throws SQLException
+    {
+        mDoDisconnect(true);
+        if (mUser == null && mPassword == null)
+            mConn = DriverManager.getConnection(mUrl, mProperties);
+        else
+            mConn = DriverManager.getConnection(mUrl, mUser, mPassword);
+    }
 
     /**
      * Closes the current set of Statements and the JDBC connection.
@@ -416,7 +484,7 @@ class Connection implements Runnable
      */
     private void mDoDisconnect(boolean shouldCloseConnection)
     {
-        if (mStatementTable != null)
+        if (mStatementTable.size() > 0)
         {
             java.util.Enumeration elements = mStatementTable.elements();
             while (elements.hasMoreElements())
@@ -426,8 +494,14 @@ class Connection implements Runnable
                 gLog.debug("Closed " + mStatementTable.size() + 
                     " Statements on disconnect");
             }
-            mStatementTable = null;
         }
+        // Don't empty the handle cache. Leave older statement
+        // handles around for destroy requests in case we're in a
+        // reconnect situation. Destroy shouldn't throw
+        // exceptions back to the Perl client even if the handle
+        // has already been closed.  
+
+        // mStatementTable.clear();
         if (mConn != null && shouldCloseConnection)
         {
             try
@@ -497,8 +571,27 @@ class Connection implements Runnable
     BerObject handleRequest(PrepareRequest aRequest)
         throws SQLException
     {
-        PreparedStatement stmt = 
-            mConn.prepareStatement(aRequest.getStatement());
+        PreparedStatement stmt = null;
+        if (mSupportsGetGeneratedKeys)
+        {
+            if ("name".equals(aRequest.getKeyType()))
+            {
+                stmt = mConn.prepareStatement(aRequest.getStatement(), 
+                    aRequest.getColumnNames()); 
+            }
+            else if ("index".equals(aRequest.getKeyType()))
+            {
+                stmt = mConn.prepareStatement(aRequest.getStatement(),
+                    aRequest.getColumnIndexes()); 
+            }
+            else
+            {
+                stmt = mConn.prepareStatement(aRequest.getStatement(),
+                    Statement.RETURN_GENERATED_KEYS);
+            }
+        }
+        else
+            stmt = mConn.prepareStatement(aRequest.getStatement());
         int stmtHandle = mNextHandle++;
         mStatementTable.put(new Integer(stmtHandle), 
             new StatementHolder(stmt));
@@ -596,9 +689,9 @@ class Connection implements Runnable
                 case Types.LONGVARCHAR:  // Use a stream here?
                     stmt.setString(i + 1, params[i].value.toString());
                     break;
-                case Types.DATE:      // ??? Perhaps we could accept
-                case Types.TIME:      // ??? dates in the default JDBC
-                case Types.TIMESTAMP: // ??? format.
+                case Types.DATE:      
+                case Types.TIME:      
+                case Types.TIMESTAMP: 
                     // TODO: Documentation in the Changes file for DBI
                     // 1.41 indicates that if a type was
                     // explicitly supplied, we can assume the
@@ -666,6 +759,33 @@ class Connection implements Runnable
             gLog.debug("Getting and returning a row count");
             resp = new ExecuteResponse(new ExecuteRowsResponse(
                 stmt.getUpdateCount()));
+            if (mSupportsGetGeneratedKeys)
+            {
+                ResultSet rs = stmt.getGeneratedKeys();
+                ResultSetMetaData rsmd = rs.getMetaData(); 
+                if (rsmd.getColumnCount() > 0)
+                {
+                    // We might not be able to jump to the last
+                    // row, so we just have to read each row,
+                    // letting the last row be the last value
+                    // cached.
+                    mGeneratedKeys = new GeneratedKey[rsmd.getColumnCount()]; 
+                    while (rs.next())
+                    {
+                        for (int i = 1; i <= rsmd.getColumnCount(); i++)
+                        {
+                            mGeneratedKeys[i - 1] = new GeneratedKey(
+                                rsmd.getCatalogName(i),
+                                rsmd.getSchemaName(i),
+                                rsmd.getTableName(i),
+                                rsmd.getColumnName(i),
+                                rs.getString(i)); 
+                            if (gLog.isTraceEnabled())
+                                gLog.trace("Key: " + mGeneratedKeys[i - 1]); 
+                        }
+                    }
+                }
+            }
         }
         return resp;
     }
@@ -868,8 +988,7 @@ class Connection implements Runnable
             {
                 throw new UnreachableCodeException();
             }
-            
-        }
+        }            
         throw new DbdException(DbdException.gUNKNOWN_PROPERTY, 
             new String[] { property });
     }
@@ -888,7 +1007,9 @@ class Connection implements Runnable
         String property = aRequest.getPropertyName();
         if (property.equals("AutoCommit"))
         {
-            mConn.setAutoCommit(aRequest.getPropertyValue().equals("1"));
+            boolean autoCommit = aRequest.getPropertyValue().equals("1"); 
+            gLog.debug("Settting AutoCommit to " + autoCommit); 
+            mConn.setAutoCommit(autoCommit);
             return new SetConnectionPropertyResponse();
         }
         throw new DbdException(DbdException.gUNKNOWN_PROPERTY, 
@@ -1000,7 +1121,7 @@ class Connection implements Runnable
                     case ResultSetMetaData.columnNullableUnknown:
                         data[i - 1] = sDbiNullableUnknown;
                         break;
-                    default:  // UnreachableCodeException???
+                    default:
                         data[i - 1] = null;
                         gLog.warn("isNullable returned an unknown value " + 
                             nullable);
@@ -1108,7 +1229,7 @@ class Connection implements Runnable
         int handle = aRequest.getHandle();
         StatementHolder holder = (StatementHolder) mStatementTable.remove(
             new Integer(handle));
-        // ??? Should this be a runtime exception? Can the user cause this?
+        // XXX: Should this be a runtime exception? Can the user cause this?
         if (holder == null)
             throw new DbdException(DbdException.gINVALID_STATEMENT_HANDLE);
         else
@@ -1239,6 +1360,88 @@ class Connection implements Runnable
     }
 
 
+    /**
+     * Returns the cached key that best matches the request.
+     *
+     * @param aRequest may contain a table and column name to use
+     * in key lookup
+     * @return a response packet containing a key; may be the empty string
+     * @exception DbdException if an error occurs
+     */
+    /* We're ignoring the catalog and schema values. */
+    BerObject handleRequest(GetGeneratedKeysRequest aRequest) 
+        throws DbdException
+    {
+        try
+        {
+            String table = aRequest.getTable();
+            String column = aRequest.getColumn(); 
+            String key = ""; 
+            if (mGeneratedKeys == null || mGeneratedKeys.length == 0)
+            {                
+                if (gLog.isDebugEnabled())
+                    gLog.debug("Generated key requested, but no keys are available"); 
+            }
+            else if (table == null && column == null)
+            {
+                // No particular table or column requested; return the first key. 
+                key = mGeneratedKeys[0].value; 
+                if (gLog.isTraceEnabled())
+                    gLog.trace("No specific key column or table requested"); 
+            }
+            else if (table == null && column != null)
+            {
+                for (int i = 0; i < mGeneratedKeys.length; i++)
+                {
+                    if (column.equalsIgnoreCase(mGeneratedKeys[i].columnName))
+                    {
+                        key = mGeneratedKeys[i].value;
+                        break;
+                    }
+                }
+                if (gLog.isTraceEnabled())
+                    gLog.trace("Key column '" + column + "' requested"); 
+            }
+            else if (table != null && column == null)
+            {
+                for (int i = 0; i < mGeneratedKeys.length; i++)
+                {
+                    if (table.equalsIgnoreCase(mGeneratedKeys[i].table))
+                    {
+                        key = mGeneratedKeys[i].value;
+                        break;
+                    }
+                }
+                if (gLog.isTraceEnabled())
+                    gLog.trace("Key table '" + table + "' requested"); 
+            }
+            else if (table != null && column != null)
+            {
+                for (int i = 0; i < mGeneratedKeys.length; i++)
+                {
+                    if (column.equalsIgnoreCase(mGeneratedKeys[i].columnName) &&
+                        table.equalsIgnoreCase(mGeneratedKeys[i].table))
+                    {
+                        key = mGeneratedKeys[i].value;
+                        break;
+                    }
+                }
+                if (gLog.isTraceEnabled())
+                {
+                    gLog.trace("Key table and column '" + table + "'.'" + 
+                        column + "' requested"); 
+                }
+            }
+            return new GetGeneratedKeysResponse(key,
+                mBerModule.getCharacterEncoding()); 
+        }
+        catch (UnsupportedEncodingException unsupEnc)
+        { 
+            throw new DbdException(DbdException.gUNSUPPORTED_ENCODING,
+                new String[] { mBerModule.getCharacterEncoding() });
+        }
+    }
+
     /**
      * Uses reflection to call the given method on the given
      * object with the given parameter list.
@@ -1367,83 +1570,44 @@ class Connection implements Runnable
             
             try 
             {
-                // Look up the method and call it. If the
-                // implementation class isn't public, we may need
-                // to look for the public interface which specified
-                // the method.
-                Class c = anObject.getClass();
+                // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6342411
+                // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4283544
+                // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4071957
+
+                // Class.getMethod claims to get the public
+                // method requested from the class or a
+                // superclass or an interface. There's a bug
+                // which prevents this from working in some
+                // cases. One such case can be where a JDBC
+                // implementation class implements a JDBC
+                // interface, say, ResultSet, but the
+                // implementation class is not public. Using
+                // reflection to get e.g. the "close" method on
+                // the implementation class will fail. You have
+                // to get the ResultSet interface first. It's an
+                // IllegalAccessException problem, not a
+                // NoSuchMethodException problem.
+
                 if (aParameterList.length == 0)
                     parameterClasses = null;
-                Method method = c.getMethod(aMethodName, parameterClasses);
-                int mod = c.getModifiers();
-                if (!Modifier.isPublic(mod))
-                {
-                    if (gLog.isTraceEnabled())
-                    {
-                        gLog.trace(c.getName() + " is not public; looking " +
-                            "for a public interface with method " + 
-                            aMethodName);
-                    }
-                    Class[] interfaces = c.getInterfaces(); 
-                    for (int i = 0; i < interfaces.length; i++)
-                    {
-                        mod = interfaces[i].getModifiers();
-                        if (Modifier.isPublic(mod))
-                        {
-                            try
-                            {
-                                method = interfaces[i].getMethod(
-                                    aMethodName, parameterClasses);
-                                if (gLog.isTraceEnabled())
-                                {
-                                    gLog.trace("Found public interface " +
-                                        interfaces[i].getName() + 
-                                        " with method " + aMethodName);
-                                }
-                                break;
-                            }
-                            catch (NoSuchMethodException e)
-                            {
-                            }
-                            catch (SecurityException e)
-                            {
-                            }
-                        }
-                    }
-                }
-                
-                if (gLog.isTraceEnabled())
-                    gLog.trace("Found method: " + method.toString());
-                Object returnValue = null;
-                returnValue = method.invoke(anObject, parameterObjects ); 
+                Object returnValue = mInvokeMethod(anObject,
+                    aMethodName, parameterClasses, parameterObjects); 
+
                 // Null should be turned into BerNull. Boolean is
                 // turned into 1 or 0 so that the return value
                 // will be appropriately true or false in Perl.
                 if (returnValue == null)
-                {
-                    //System.out.println("returning null");
                     return null;
-                }
                 else if (returnValue instanceof Boolean)
                     return ((Boolean) returnValue).booleanValue() ? "1" : "0";
                 else
                     return returnValue.toString();
             }
-            catch (NoSuchMethodException noMethod)
-            {
-                throw new DbdException(DbdException.gREFLECTION_EXCEPTION,
-                    new String[] { noMethod.toString() });
-            }
             catch (SecurityException security)
             {
                 throw new DbdException(DbdException.gREFLECTION_EXCEPTION,
                     new String[] { security.toString() });
             }
-            catch (IllegalAccessException access)
-            {
-                throw new DbdException(DbdException.gREFLECTION_EXCEPTION,
-                    new String[] { access.toString() });
-            }
             catch (IllegalArgumentException argument)
             {
                 throw new DbdException(DbdException.gREFLECTION_EXCEPTION,
@@ -1465,11 +1629,126 @@ class Connection implements Runnable
         }
         catch (Exception e) 
         { 
+            gLog.debug(e, e); 
             throw new DbdException(DbdException.gGENERIC_EXCEPTION,
                 new String[] { e.toString() });
         }            
     }
 
+    /**
+     * Attempts to invoke a method on the given object.
+     *
+     * @param anObject the object
+     * @param aMethodName the method name
+     * @param parameterClasses the class objects for the method parameters
+     * @param parameterObjects the parameter objects
+     * @return the method return value    
+     * @throws IllegalArgumentException
+     * @throws InvocationTargetException
+     * @throws DbdException if an otherwise unreported problem
+     * causes the method to go uninvoked
+     */
+    private Object mInvokeMethod(Object anObject, String aMethodName, 
+        Class[] parameterClasses, Object[] parameterObjects) 
+        throws IllegalArgumentException, InvocationTargetException,
+        DbdException
+    {
+        Class currentClass = anObject.getClass(); 
+        while (currentClass != null)
+        {
+            try
+            {
+                Method method = currentClass.getMethod(aMethodName, 
+                    parameterClasses);
+                gLog.trace("Invoking " + currentClass.getName() + "." + 
+                    aMethodName); 
+                return method.invoke(anObject, parameterObjects);
+            }
+            catch (NoSuchMethodException noMethod)
+            {
+                gLog.trace(currentClass.getName() + "/" + noMethod.toString()); 
+            }
+            catch (SecurityException security)
+            {
+                gLog.trace(currentClass.getName() + "/" + security.toString()); 
+            }
+            catch (IllegalAccessException access)
+            {
+                gLog.trace(currentClass.getName() + "/" + access.toString()); 
+            }
+
+            Class[] interfaces = currentClass.getInterfaces(); 
+            for (int i = 0; i < interfaces.length; i++)
+            {
+                try
+                {
+                    Method method = interfaces[i].getMethod(
+                        aMethodName, parameterClasses);
+                    gLog.trace("Invoking " + interfaces[i].getName() + "." + 
+                        aMethodName); 
+                    return method.invoke(anObject, parameterObjects); 
+                }
+                catch (NoSuchMethodException noMethod)
+                {
+                    gLog.trace(currentClass.getName() + "/" + 
+                        noMethod.toString()); 
+                }
+                catch (SecurityException security)
+                {
+                    gLog.trace(currentClass.getName() + "/" + 
+                        security.toString()); 
+                }
+                catch (IllegalAccessException access)
+                {
+                    gLog.trace(currentClass.getName() + "/" + 
+                        access.toString()); 
+                }
+            }
+
+            currentClass = currentClass.getSuperclass();
+        }
+        throw new DbdException(DbdException.gREFLECTION_EXCEPTION,
+            new String[] { "Unable to invoke method" }); 
+    }
+
+    /*
+    private Method mTestInterfaces(Class c, String aMethodName, 
+        Class[] parameterClasses) 
+    {
+        Class[] interfaces = c.getInterfaces(); 
+        gLog.debug("Found " + interfaces.length + 
+            " other interfaces of " + c.getName() + " to try"); 
+        for (int i = 0; i < interfaces.length; i++)
+        {
+            int mod = interfaces[i].getModifiers();
+            if (Modifier.isPublic(mod))
+            {
+                gLog.debug("interface " + interfaces[i].getName() +
+                    " is public"); 
+                try
+                {
+                    Method method = interfaces[i].getMethod(
+                        aMethodName, parameterClasses);
+                    if (gLog.isTraceEnabled())
+                    {
+                        gLog.trace("Found public interface " +
+                            interfaces[i].getName() + 
+                            " with method " + aMethodName);
+                    }
+                    return method;
+                }
+                catch (NoSuchMethodException e)
+                {
+                }
+                catch (SecurityException e)
+                {
+                }
+            }
+        }
+        return null; 
+    }
+    */
+
 
     // Support methods.
 
@@ -1542,7 +1821,7 @@ class Connection implements Runnable
     {
         StatementHolder holder = (StatementHolder) mStatementTable.get(
             new Integer(aStatementHandle));
-        // ??? Should this be a runtime exception? Can the user cause this?
+        // XXX: Should this be a runtime exception? Can the user cause this?
         if (holder == null)
             throw new DbdException(DbdException.gINVALID_STATEMENT_HANDLE);
         return holder;
@@ -1628,7 +1907,6 @@ class Connection implements Runnable
         return baos.toByteArray();
     }
 
-
     /**
      * Reads a LONG field. Implements the DBI semantics associated with
      * the LongTruncOk and LongReadLen properties.
@@ -1843,3 +2121,75 @@ class StatementHolder
     }
 }
 
+
+/**
+ * Caches the value and metadata for a generated key.
+ */
+class GeneratedKey
+{
+    /** The catalog name. */
+    String catalog;
+
+    /** The schema name. */
+    String schema;
+
+    /** The table name. */
+    String table;
+
+    /** The column name. */
+    String columnName;
+
+    /** The key value. */
+    String value;
+
+
+    /**
+     * Constructor - caches values.
+     *
+     * @param aCatalog the catalog
+     * @param aSchema the schema
+     * @param aTable the table
+     * @param aColumnName the column name
+     * @param aValue the value
+     */
+    GeneratedKey(String aCatalog, String aSchema, String aTable, 
+        String aColumnName, String aValue)
+    {
+        catalog = aCatalog;
+        schema = aSchema;
+        table = aTable;
+        columnName = aColumnName;
+        value = aValue;
+    }
+
+
+    /**
+     * Returns a string representation of the key.
+     *
+     * @return a string representation of the key
+     */
+    public String toString()
+    {
+        StringBuffer b = new StringBuffer();
+        if (catalog != null)
+            b.append(catalog);
+        if (schema != null && b.length() > 0)
+            b.append(".").append(schema);
+        else if (schema != null)
+            b.append(schema);            
+        if (table != null && b.length() > 0)
+            b.append(".").append(table);
+        else if (table != null)
+            b.append(table);            
+        if (columnName != null && b.length() > 0)
+            b.append(".").append(columnName);
+        else if (columnName != null)
+            b.append(columnName);
+        b.append(" = "); 
+        if (value != null)
+            b.append(value);
+        else
+            b.append("NULL");
+        return b.toString(); 
+    }
+}
