@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2001 Vizdom Software, Inc. All Rights Reserved.
+ * Copyright 1999-2002,2006 Vizdom Software, Inc. All Rights Reserved.
  * 
  *  This program is free software; you can redistribute it and/or 
  *  modify it under the same terms as the Perl Kit, namely, under 
@@ -22,11 +22,11 @@ package com.vizdom.dbd.jdbc;
 
 import com.vizdom.ber.*;
 
+
 /**
  * A request to set the value of a statement property.
  *
- * @author Gennis Emerson
- * @version $Revision: 1.9 $
+ * @version $Revision: 1.10 $
  */
 class SetStatementPropertyRequest extends BerSequence
 {
@@ -50,6 +50,7 @@ class SetStatementPropertyRequest extends BerSequence
         }
     };
 
+
     /**
      * The decoding constructor.
      */
@@ -58,6 +59,7 @@ class SetStatementPropertyRequest extends BerSequence
         super();
     }    
 
+
     /**
      * Returns the statement handle identifying the statement whose
      * property should be changed.
@@ -80,6 +82,7 @@ class SetStatementPropertyRequest extends BerSequence
         return ((BerOctetString) mSequence[1]).toString();
     }
 
+
     /**
      * Returns the new property value
      *
@@ -90,6 +93,7 @@ class SetStatementPropertyRequest extends BerSequence
         return ((BerOctetString) mSequence[2]).toString();
     }
 
+
     /**
      * Returns the identifier for this BerObject. 
      *
@@ -100,6 +104,7 @@ class SetStatementPropertyRequest extends BerSequence
         return gIDENTIFIER;
     }
 
+
     /**
      * Returns a string representation of this object.
      *
@@ -107,7 +112,7 @@ class SetStatementPropertyRequest extends BerSequence
      */
     public String toString()
     {
-        return "Set connection property: " + getPropertyName() + "=" +
+        return "Set statement property: " + getPropertyName() + "=" +
             getPropertyValue();
     }
 }
