@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2004 Vizdom Software, Inc. All Rights Reserved.
+ * Copyright 1999-2005,2008 Vizdom Software, Inc. All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or 
  * modify it under the same terms as the Perl Kit, namely, under 
@@ -22,7 +22,6 @@ package com.vizdom.util;
 
 import java.util.Hashtable;
 import com.vizdom.util.UnreachableCodeException;
-import org.apache.log4j.Level;
 import org.apache.log4j.Logger;
 
 /**
@@ -36,7 +35,7 @@ import org.apache.log4j.Logger;
  * to need.
  *   
  * @author Gennis Emerson
- * @version $Revision: 1.28 $
+ * @version $Revision: 1.29 $
  */
 public abstract class CharacterEncoder
 {
@@ -62,9 +61,10 @@ public abstract class CharacterEncoder
      */
     private static String gLatin1Encoding = null;
 
-    // All supported non-Java encodings must be listed here.
+
     static
     {
+        // All supported non-Java encodings must be listed here.
         gEncoders = new Hashtable();
         gEncoders.put("DEC_MCS", gPLACE_HOLDER);
         gEncoders.put("HP_Roman8", gPLACE_HOLDER);
@@ -931,7 +931,7 @@ class CharacterEncoderWriter extends java.io.Writer
      * @exception java.io.IOException if an error occurs writing to 
      *      the underlying stream
      */
-    public void write(char aCharacterBuffer[], int anOffset, int aLength) 
+    public void write(char[] aCharacterBuffer, int anOffset, int aLength) 
         throws java.io.IOException
     {
         mEnsureValid();
@@ -1035,7 +1035,7 @@ class CharacterEncoderReader extends java.io.Reader
      * @exception java.io.IOException if an error occurs reading from
      *      the underlying stream
      */
-    public int read(char aCharacterBuffer[], int anOffset, int aLength) 
+    public int read(char[] aCharacterBuffer, int anOffset, int aLength) 
         throws java.io.IOException
     {
         mEnsureValid();
