@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2004 Vizdom Software, Inc. All Rights Reserved.
+ * Copyright 1999-2004,2008 Vizdom Software, Inc. All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or 
  * modify it under the same terms as the Perl Kit, namely, under 
@@ -32,7 +32,7 @@ import java.io.Writer;
  * A collection of helpful methods for debugging and assertions.
  * 
  * @author John Lacey
- * @version $Revision: 1.32 $
+ * @version $Revision: 1.33 $
  */
 public class Debug
 {
@@ -82,7 +82,7 @@ public class Debug
     private static final PrintWriter gNullLogWriter = 
         new PrintWriter(System.err) {
             public void write(int b) {}
-            public void write(char buf[], int off, int len) {}
+            public void write(char[] buf, int off, int len) {}
             public void write(String s, int off, int len) {}
             public void println() {}
             public void println(boolean b) {}
@@ -109,7 +109,7 @@ public class Debug
     private static final PrintStream gNullLogStream = 
         new PrintStream(System.err) {
             public void write(int b) {}
-            public void write(byte buf[], int off, int len) {}
+            public void write(byte[] buf, int off, int len) {}
             public void println() {}
             public void println(boolean b) {}
             public void println(char c) {}
@@ -122,6 +122,7 @@ public class Debug
             public void println(String s) {}
         };
 
+
     static
     {
         PrintStream defaultStream = System.err;
